--[[
	nowagepayment.lua
	--- IN DEVELOPMENT ---
	version: 0.1.2.1
	date: 21/12/19
	author: BigBlueHH
]]

NoWagePayment = {}
NoWagePayment.aiCostsPerRealHour = 1000
NoWagePayment.enteredVehicleIDs = {}

function NoWagePayment.prerequisitesPresent(specializations)
	return true
end

function NoWagePayment.registerEventListeners(vehicleType)
  for _,n in pairs( { "onEnterVehicle","onLeaveVehicle" } ) do
		SpecializationUtil.registerEventListener(vehicleType, n, NoWagePayment)
	end
end

function NoWagePayment:loadMap(name)
  AIJob.getPricePerMs = Utils.overwrittenFunction(AIJob.getPricePerMs, NoWagePayment.getPricePerMs)
  AIJobFieldWork.getPricePerMs = Utils.overwrittenFunction(AIJobFieldWork.getPricePerMs, NoWagePayment.getPricePerMs)
  AIJobConveyor.getPricePerMs = Utils.overwrittenFunction(AIJobConveyor.getPricePerMs, NoWagePayment.getPricePerMs)
  AbstractFieldMission.finish = Utils.appendedFunction(AbstractFieldMission.finish, NoWagePayment.finishFieldMission)
end

function NoWagePayment:onEnterVehicle()
	NoWagePayment.enteredVehicleIDs[self.id] = true
end

function NoWagePayment:onLeaveVehicle()
	NoWagePayment.enteredVehicleIDs[self.id] = nil
end

function NoWagePayment:getPricePerMs(sFunc)
	if NoWagePayment.enteredVehicleIDs[self.vehicleParameter.vehicleId] ~= nil then
		return 0
	else
		if NoWagePayment.aiCostsPerRealHour == 0 then
			return sFunc()
		else
			local pricePerMs = NoWagePayment.aiCostsPerRealHour/1000/60/60
			return pricePerMs
		end
	end	
end

function NoWagePayment:finishFieldMission(success)
  if success then
    for _,j in ipairs(self.mission.aiSystem.activeJobs) do
      if self.farmId == j.startedFarmId then
        for _,v in ipairs(g_currentMission.vehicles) do
          if NoWagePayment:isAIActiveOnField(j, v, self.field) then
            v:stopCurrentAIJob(AIMessageSuccessFinishedJob.new())
          end
        end
      end
    end
  end
end

function NoWagePayment:isAIActiveOnField(j, v, f)
  if j.vehicleParameter.vehicleId == v.id and v.ownerFarmId == j.startedFarmId and v.getIsAIActive ~= nil and v:getIsAIActive() then
    for _, c in ipairs(v.components) do
      local wx, wy, wz = localToWorld(c.node, getCenterOfMass(c.node))
      local isField, _, _ = FSDensityMapUtil.getFieldDataAtWorldPosition(wx, wy, wz);
      if isField and g_farmlandManager:getFarmlandIdAtWorldPosition(wx, wz) == f.farmland.id then
        for _,p in ipairs(f.setFieldStatusPartitions) do
          if MathUtil.vector2Length(wx-p.x0,wz-p.z0) <= math.max(p.widthX,p.widthZ,p.heightX,p.heightZ) then
            return true
          end
        end
      end
    end
  end
  return false
end

addModEventListener(NoWagePayment)